{% extends 'migrator_base.html' %}

{% block title %}Delete config{% endblock %}
{% block content %}
    <form method="post">{% csrf_token %}
        <p>Are you sure you want to delete "{{ object }}"?</p>
        <input class="btn btn-danger" type="submit" value="Confirm">
    </form>
{% endblock %}