# Generated by Django 3.2.4 on 2023-07-19 06:59

from django.db import migrations, models

import testrail_megamigrator.models


class Migration(migrations.Migration):

    dependencies = [
        ('testrail_megamigrator', '0004_auto_20230411_1856'),
    ]

    operations = [
        migrations.AddField(
            model_name='testrailsettings',
            name='custom_fields_matcher',
            field=models.JSONField(default=testrail_megamigrator.models.default_custom_fields),
        ),
    ]
