# Generated by Django 3.2.4 on 2023-03-24 09:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('testrail_megamigrator', '0002_rename_dumpfile_path_testrailsettings_testy_attachments_url'),
    ]

    operations = [
        migrations.RenameField(
            model_name='testrailsettings',
            old_name='login',
            new_name='preset_name',
        ),
        migrations.RemoveField(
            model_name='testrailsettings',
            name='password',
        ),
    ]
