{% load static %}
<!doctype html>
<html lang="en-us">
<head>
    {% block stylesheet %}
        <link href="{% static 'testrail_megamigrator/css/bootstrap.min.css' %}" rel="stylesheet"/>
        <script src="{% static 'testrail_megamigrator/js/bootstrap.min.js' %}"></script>
        <script src="{% static 'testrail_megamigrator/js/prettify_json.js' %}"></script>
        <meta charset="utf-8">
    {% endblock %}
    <title>{% block title %}Testrail migrator{% endblock %}</title>
</head>
<body>
<nav class="navbar navbar-expand-lg navbar-light bg-light">
    <div class="container-fluid">
        <a class="navbar-brand" href="{% url 'plugins:testrail_megamigrator:settings-list' %}">Testrail migrator</a>
        <div class="collapse navbar-collapse" id="navbarNavDropdown">
            <ul class="navbar-nav">
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'plugins:testrail_megamigrator:settings-list' %}">Migrator settings</a>
                </li>
                <li class="nav-item dropdown">
                    <a class="nav-link" href="#" role="button" data-bs-toggle="dropdown"
                       aria-expanded="false">
                        Download objects
                    </a>
                    <ul class="dropdown-menu">
                        <li>
                            <a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:download-project' %}">Download
                                testrail projects</a>
                        </li>
                        <li><a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:download-milestones' %}">Download
                            testrail milestones</a>
                        </li>
                        <li>
                            <a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:download-suites' %}">Download
                                testrail suites</a>
                        </li>
                        <li>
                            <a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:download-plans' %}">Download
                                testrail plans/runs</a>
                        </li>
                    </ul>
                </li>
                <li class="nav-item dropdown">
                    <a class="nav-link" href="#" role="button" data-bs-toggle="dropdown"
                       aria-expanded="false">
                        Upload objects
                    </a>
                    <ul class="dropdown-menu">
                        <li>
                            <a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:upload-project' %}">Upload
                                testrail projects</a>
                        </li>
                        <li><a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:upload-milestones' %}">Upload
                            testrail milestones</a>
                        </li>
                        <li>
                            <a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:upload-suites' %}">Upload
                                testrail suites</a>
                        </li>
                        <li>
                            <a class="dropdown-item" href="{% url 'plugins:testrail_megamigrator:upload-plans' %}">Upload
                                testrail plans/runs</a>
                        </li>
                    </ul>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'plugins:testrail_megamigrator:backup-list' %}">
                        Downloaded testrail projects
                    </a>
                </li>
            </ul>
        </div>
    </div>
</nav>
<div class="w-50 mx-auto">
    {% block content %}
    {% endblock %}
</div>
</body>
</html>